package org.wasabi.interceptors

import org.wasabi.protocol.http.Request
import org.wasabi.protocol.http.Response
import org.wasabi.serializers.Serializer
import org.wasabi.app.AppServer
import org.wasabi.routing.InterceptOn
import org.wasabi.protocol.http.StatusCodes


public class ContentNegotiationInterceptor(val serializers: List<Serializer>): Interceptor() {
    override fun intercept(request: Request, response: Response): Boolean {
        var executeNext = false
            if (response.negotiatedMediaType == "" && (response.sendBuffer != null) && !(response.sendBuffer is String)) {
                for (requestedContentType in response.requestedContentTypes) {
                    val serializer = serializers.firstOrNull { it.canSerialize(requestedContentType) }
                    if (serializer != null) {
                        response.negotiatedMediaType = requestedContentType
                        executeNext = true
                        break;
                    }
                }
                if (response.negotiatedMediaType == "") {
                    response.setStatus(StatusCodes.UnsupportedMediaType)
                }
            } else {
                executeNext = true
            }

        return executeNext
    }
}

public fun AppServer.enableContentNegotiation() {
    intercept(ContentNegotiationInterceptor(serializers), "*", InterceptOn.PostExecution)
}
